"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.call_dest = exports.call_src = exports.test_dest = exports.test_src = exports.deploy = void 0;
const settings_1 = require("./settings");
const node_fetch_1 = __importStar(require("node-fetch"));
const deployer_1 = require("./deployer");
async function deploy(access_token, work_slug, work_id) {
    let settings = await settings_1.load_settings(settings_1.get_settings_file());
    if (settings.fn_type == 'source') {
        await deployer_1.deploy_source(settings, access_token, work_slug, work_id);
    }
    else {
        await deployer_1.deploy_destination(settings, access_token, work_slug, work_id);
    }
}
exports.deploy = deploy;
function urlArgsToString(urlArgs) {
    if (!urlArgs) {
        return '';
    }
    let keys = Object.keys(urlArgs);
    if (keys.length == 0) {
        return '';
    }
    let s = '?';
    keys.forEach(key => {
        let value = urlArgs[key];
        let value_str = value;
        s += '&' + key + '=' + value_str;
    });
    return s;
}
function test_src(payload, sam_port, sam_host) {
    if (typeof payload.body !== 'string') {
        payload.headers['Content-Type'] = 'application/json';
    }
    return node_fetch_1.default(`http://${sam_host || '127.0.0.1'}:${sam_port}/function${urlArgsToString(payload.queryParameters)}`, {
        method: 'post',
        body: typeof payload.body == 'string' ? payload.body : JSON.stringify(payload.body),
        headers: payload.headers
    }).then(r => r.json());
}
exports.test_src = test_src;
async function test_dest(payload, sam_port, sam_host) {
    return node_fetch_1.default(`http://${sam_host || '127.0.0.1'}:${sam_port}/function`, {
        method: 'post',
        body: JSON.stringify(payload),
        headers: {
            'Content-Type': 'application/json'
        }
    });
}
exports.test_dest = test_dest;
async function call_src(payload, onRequest, output) {
    output.tracks = [];
    output.identifies = [];
    output.groups = [];
    output.aliases = [];
    output.screens = [];
    output.pages = [];
    global.Segment = {
        identify(i) {
            output.identifies.push(i);
        },
        track(t) {
            output.tracks.push(t);
        },
        group(t) {
            output.groups.push(t);
        },
        alias(t) {
            output.aliases.push(t);
        },
        page(t) {
            output.pages.push(t);
        },
        screen(t) {
            output.screens.push(t);
        }
    };
    let body = payload.body;
    const settings = {};
    onRequest({
        json() {
            return JSON.parse(body);
        },
        text() {
            return body;
        },
        headers: new node_fetch_1.Headers(payload.headers),
        // TODO: This mimics the Segment Test approach. I suspect no one uses this in reality, but does this need to match the actual URL called by the test?
        url: new URL('https://fn.segmentapis.com/' + urlArgsToString(payload.queryStringParameters))
    }, settings);
}
exports.call_src = call_src;
async function call_dest(_event, fns) {
    let event = JSON.parse(_event.body);
    const settings = {};
    switch (event.type) {
        case 'alias': {
            if (fns.onAlias) {
                fns.onAlias(event, settings);
            }
            break;
        }
        case 'identify': {
            if (fns.onIdentify) {
                fns.onIdentify(event, settings);
            }
            break;
        }
        case 'page': {
            if (fns.onPage) {
                fns.onPage(event, settings);
            }
            break;
        }
        case 'screen': {
            if (fns.onScreen) {
                fns.onScreen(event, settings);
            }
            break;
        }
        case 'track': {
            if (fns.onTrack) {
                fns.onTrack(event, settings);
            }
            break;
        }
        case 'group': {
            if (fns.onGroup) {
                fns.onGroup(event, settings);
            }
            break;
        }
    }
}
exports.call_dest = call_dest;
//# sourceMappingURL=index.js.map