import { exec, spawn } from 'child_process'
import { Settings } from './settings'

export function nodemon_installed(): Promise<boolean> {
  return new Promise((resolve, reject) => {
    exec(`nodemon --version`, (error, std_out) => {
      if (error) {
        reject(error)
      } else {
        resolve(std_out.substring(0, 1).match(/\d/) !== null)
      }
    })
  })
}

export async function run_exec(settings: Settings, is_debug: boolean) {
  if (settings.language == 'typescript') {
    let is_nodemon = await nodemon_installed()
    let te = is_nodemon ? `nodemon --watch src -e ts --exec "tsc -p ."` : 'tsc -p .'
    spawn(te, { shell: true, stdio: 'inherit' })
  }

  let e = `sam local start-api --port ${settings.sam_port}`
  if (is_debug) {
    e = `${e} --debug-port ${settings.debug_port}`
  }

  spawn(e, { shell: true, stdio: 'inherit' })
}