import MemoryFS from 'memory-fs'
import webpack from 'webpack'

export function pack(code: string, is_src: boolean): Promise<string> {
  mem_fs.writeFileSync('/in.js', code, 'utf8')

  let compiler = webpack({
    mode: 'none',
    entry: {
      app: '/in.js'
    },

    externals: ['aws-sdk', 'lodash', 'atob', 'btoa', 'form-data', 'crypto', 'events', 'stream', 'node-fetch', 'oauth', 'xml'],
    output: {
      libraryTarget: 'var',
      library: 'webpack',
      path: '/',
      filename: './out.js'
    }
  })

  compiler.inputFileSystem = mem_fs
  compiler.outputFileSystem = mem_fs

  return new Promise((resolve, reject) => {
    compiler.run((err) => {
      if (err) {
        reject(err)
      } else {
        resolve(`var lodash = _;
var aws_sdk = AWS;
var form_data = FormData;
var oauth = OAuth;
        ` + mem_fs.readFileSync('/out.js', 'utf8')
            .replace(/module\.exports \= aws\-sdk/, 'module.exports = aws_sdk')
            .replace(/module\.exports \= form\-data/, 'module.exports = form_data') + is_src ? `
onRequest = webpack['onRequest']` : `
onTrack = webpack['onTrack']
onAlias = webpack['onAlias']
onIdentify = webpack['onIdentify']
onGroup = webpack['onGroup']
onPage = webpack['onPage']
onScreen = webpack['onScreen']`)
      }
    })
  })
}

const mem_fs = new MemoryFS()
